@@ -1,8 +1,8 @@
-/* Copyright © 2005-2007  Roger Leigh <rleigh@debian.org>
+/* Copyright © 2005-2008  Roger Leigh <rleigh@debian.org>
  *
- * schroot is free software; you can redistribute it and/or modify it
+ * schroot is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * schroot is distributed in the hope that it will be useful, but
@@ -11,9 +11,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
- * MA  02111-1307  USA
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  *
  *********************************************************************/
 
@@ -22,15 +21,11 @@
 #include "sbuild-chroot-block-device.h"
 #include "sbuild-format-detail.h"
 #include "sbuild-lock.h"
+#include "sbuild-util.h"
 
 #include <cerrno>
 #include <cstring>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/sysmacros.h>
-#include <unistd.h>
-
 #include <boost/format.hpp>
 
 using boost::format;
@@ -123,8 +118,6 @@ chroot_block_device::setup_lock (chroot::setup_type type,
 				 bool               lock,
 				 int                status)
 {
-  struct stat statbuf;
-
   /* Only lock during setup, not exec. */
   if (type == EXEC_START || type == EXEC_STOP)
     return;
@@ -134,11 +127,7 @@ chroot_block_device::setup_lock (chroot::setup_type type,
       (type == SETUP_STOP && lock == true))
     return;
 
-  if (stat(this->device.c_str(), &statbuf) == -1)
-    {
-      throw error(get_device(), DEVICE_STAT, strerror(errno));
-    }
-  else if (!S_ISBLK(statbuf.st_mode))
+  if (!stat(this->device).is_block())
     {
       throw error(get_device(), DEVICE_NOTBLOCK);
     }
@@ -173,7 +162,7 @@ chroot_block_device::setup_lock (chroot::setup_type type,
 sbuild::chroot::session_flags
 chroot_block_device::get_session_flags () const
 {
-  return static_cast<session_flags>(0);
+  return SESSION_NOFLAGS;
 }
 
 void
@@ -203,19 +192,23 @@ chroot_block_device::get_keyfile (keyfile& keyfile) const
 }
 
 void
-chroot_block_device::set_keyfile (keyfile const& keyfile)
+chroot_block_device::set_keyfile (keyfile const& keyfile,
+				  string_list&   used_keys)
 {
-  chroot::set_keyfile(keyfile);
+  chroot::set_keyfile(keyfile, used_keys);
 
   keyfile::get_object_value(*this, &chroot_block_device::set_device,
 			    keyfile, get_name(), "device",
 			    keyfile::PRIORITY_REQUIRED);
+  used_keys.push_back("device");
 
   keyfile::get_object_value(*this, &chroot_block_device::set_mount_options,
 			    keyfile, get_name(), "mount-options",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("mount-options");
 
   keyfile::get_object_value(*this, &chroot_block_device::set_location,
 			    keyfile, get_name(), "location",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("location");
 }
