@@ -1,8 +1,8 @@
-/* Copyright © 2005-2007  Roger Leigh <rleigh@debian.org>
+/* Copyright © 2005-2008  Roger Leigh <rleigh@debian.org>
  *
- * schroot is free software; you can redistribute it and/or modify it
+ * schroot is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * schroot is distributed in the hope that it will be useful, but
@@ -11,9 +11,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
- * MA  02111-1307  USA
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  *
  *********************************************************************/
 
@@ -26,11 +25,6 @@
 #include <cerrno>
 #include <iostream>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/sysmacros.h>
-#include <unistd.h>
-
 #include <boost/format.hpp>
 
 using std::endl;
@@ -125,7 +119,6 @@ chroot_lvm_snapshot::setup_lock (chroot::setup_type type,
 				 int                status)
 {
   std::string device;
-  struct stat statbuf;
 
   /* Lock is removed by setup script on setup stop.  Unlocking here
      would fail: the LVM snapshot device no longer exists. */
@@ -137,14 +130,10 @@ chroot_lvm_snapshot::setup_lock (chroot::setup_type type,
 	device = get_snapshot_device();
 
       if (device.empty())
-	{
-	  throw error(CHROOT_DEVICE);
-	}
-      else if (stat(device.c_str(), &statbuf) == -1)
-	{
-	  throw error(get_device(), DEVICE_STAT, strerror(errno));
-	}
-      else if (!S_ISBLK(statbuf.st_mode))
+	throw error(CHROOT_DEVICE);
+
+      stat file_status(device);
+      if (!file_status.is_block())
 	{
 	  throw error(get_device(), DEVICE_NOTBLOCK);
 	}
@@ -193,7 +182,7 @@ chroot_lvm_snapshot::setup_lock (chroot::setup_type type,
 sbuild::chroot::session_flags
 chroot_lvm_snapshot::get_session_flags () const
 {
-  return SESSION_CREATE;
+  return SESSION_CREATE | chroot_source::get_session_flags();
 }
 
 void
@@ -223,18 +212,21 @@ chroot_lvm_snapshot::get_keyfile (keyfile& keyfile) const
 }
 
 void
-chroot_lvm_snapshot::set_keyfile (keyfile const& keyfile)
+chroot_lvm_snapshot::set_keyfile (keyfile const& keyfile,
+				  string_list&   used_keys)
 {
-  chroot_block_device::set_keyfile(keyfile);
-  chroot_source::set_keyfile(keyfile);
+  chroot_block_device::set_keyfile(keyfile, used_keys);
+  chroot_source::set_keyfile(keyfile, used_keys);
 
   keyfile::get_object_value(*this, &chroot_lvm_snapshot::set_snapshot_device,
 			    keyfile, get_name(), "lvm-snapshot-device",
 			    get_active() ?
 			    keyfile::PRIORITY_REQUIRED :
 			    keyfile::PRIORITY_DISALLOWED);
+  used_keys.push_back("lvm-snapshot-device");
 
   keyfile::get_object_value(*this, &chroot_lvm_snapshot::set_snapshot_options,
 			    keyfile, get_name(), "lvm-snapshot-options",
 			    keyfile::PRIORITY_REQUIRED);
+  used_keys.push_back("lvm-snapshot-options");
 }
