@@ -1,8 +1,8 @@
-/* Copyright © 2005-2007  Roger Leigh <rleigh@debian.org>
+/* Copyright © 2005-2008  Roger Leigh <rleigh@debian.org>
  *
- * schroot is free software; you can redistribute it and/or modify it
+ * schroot is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * schroot is distributed in the hope that it will be useful, but
@@ -11,9 +11,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
- * MA  02111-1307  USA
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  *
  *********************************************************************/
 
@@ -27,10 +26,8 @@
 #include "sbuild-chroot-lvm-snapshot.h"
 #include "sbuild-lock.h"
 
-#include <algorithm>
 #include <cerrno>
 #include <map>
-#include <set>
 #include <utility>
 
 #include <ext/stdio_filebuf.h>
@@ -62,14 +59,12 @@ namespace
       emap(sbuild::chroot::DEVICE_ABS,      N_("Device must have an absolute path")),
       emap(sbuild::chroot::DEVICE_LOCK,     N_("Failed to lock device")),
       emap(sbuild::chroot::DEVICE_NOTBLOCK, N_("File is not a block device")),
-      emap(sbuild::chroot::DEVICE_STAT,     N_("Failed to stat device")),
       emap(sbuild::chroot::DEVICE_UNLOCK,   N_("Failed to unlock device")),
       emap(sbuild::chroot::FILE_ABS,        N_("File must have an absolute path")),
       emap(sbuild::chroot::FILE_LOCK,       N_("Failed to acquire file lock")),
       emap(sbuild::chroot::FILE_NOTREG,     N_("File is not a regular file")),
       emap(sbuild::chroot::FILE_OWNER,      N_("File is not owned by user root")),
       emap(sbuild::chroot::FILE_PERMS,      N_("File has write permissions for others")),
-      emap(sbuild::chroot::FILE_STAT,       N_("Failed to stat file")),
       emap(sbuild::chroot::FILE_UNLOCK,     N_("Failed to discard file lock")),
       emap(sbuild::chroot::LOCATION_ABS,    N_("Location must have an absolute path")),
       // TRANSLATORS: unlink refers to the C function which removes a file
@@ -394,6 +389,12 @@ sbuild::chroot::setup_env (environment& env)
   env.add("CHROOT_PATH", get_path());
   env.add("CHROOT_MOUNT_DEVICE", get_mount_device());
   env.add("CHROOT_SCRIPT_CONFIG", normalname(std::string(PACKAGE_SYSCONF_DIR) +  '/' + get_script_config()));
+  env.add("CHROOT_SESSION_CREATE",
+	  static_cast<bool>(get_session_flags() & SESSION_CREATE));
+  env.add("CHROOT_SESSION_CLONE",
+	  static_cast<bool>(get_session_flags() & SESSION_CLONE));
+  env.add("CHROOT_SESSION_PURGE",
+	  static_cast<bool>(get_session_flags() & SESSION_PURGE));
 }
 
 void
@@ -480,11 +481,17 @@ sbuild::chroot::get_details (format_detail& detail) const
     .add(_("Run Execution Scripts"), get_run_exec_scripts())
     .add(_("Script Configuration"), get_script_config())
     .add(_("Session Managed"),
-	 static_cast<bool>(get_session_flags() & chroot::SESSION_CREATE));
+	 static_cast<bool>(get_session_flags() & chroot::SESSION_CREATE))
+    .add(_("Session Cloned"),
+	 static_cast<bool>(get_session_flags() & chroot::SESSION_CLONE))
+    .add(_("Session Purged"),
+	 static_cast<bool>(get_session_flags() & chroot::SESSION_PURGE));
 
   if (!get_command_prefix().empty())
     detail.add(_("Command Prefix"), get_command_prefix());
 
+  // TRANSLATORS: "Personality" is the Linux kernel personality
+  // (process execution domain).  See schroot.conf(5).
   detail.add(_("Personality"), get_persona().get_name());
 
   /* Non user-settable properties are listed last. */
@@ -570,78 +577,109 @@ sbuild::chroot::get_keyfile (keyfile& keyfile) const
 }
 
 void
-sbuild::chroot::set_keyfile (keyfile const& keyfile)
+sbuild::chroot::set_keyfile (keyfile const& keyfile,
+			     string_list&   used_keys)
 {
+  // Keys which are used elsewhere, but should be counted as "used".
+  used_keys.push_back("type");
+
+  string_list keys = keyfile.get_keys(get_name());
+  for (string_list::const_iterator pos = keys.begin();
+       pos != keys.end();
+       ++pos)
+    {
+      static regex description_keys("^description\\[.*\\]$");
+      if (regex_search(*pos, description_keys))
+	used_keys.push_back(*pos);
+    }
+
   // This is set not in the configuration file, but set in the keyfile
   // manually.  The user must not have the ability to set this option.
   keyfile::get_object_value(*this, &chroot::set_active,
 			    keyfile, get_name(), "active",
 			    keyfile::PRIORITY_REQUIRED);
+  used_keys.push_back("active");
 
   keyfile::get_object_value(*this, &chroot::set_run_setup_scripts,
 			    keyfile, get_name(), "run-setup-scripts",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("run-setup-scripts");
 
   keyfile::get_object_value(*this, &chroot::set_run_exec_scripts,
 			    keyfile, get_name(), "run-session-scripts",
 			    keyfile::PRIORITY_DEPRECATED);
+  used_keys.push_back("run-session-scripts");
   keyfile::get_object_value(*this, &chroot::set_run_exec_scripts,
 			    keyfile, get_name(), "run-exec-scripts",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("run-exec-scripts");
 
   keyfile::get_object_value(*this, &chroot::set_script_config,
 			    keyfile, get_name(), "script-config",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("script-config");
 
   keyfile::get_object_value(*this, &chroot::set_priority,
 			    keyfile, get_name(), "priority",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("priority");
 
   keyfile::get_object_list_value(*this, &chroot::set_aliases,
 				 keyfile, get_name(), "aliases",
 				 keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("aliases");
 
   keyfile::get_object_value(*this, &chroot::set_environment_filter,
 			    keyfile, get_name(), "environment-filter",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("environment-filter");
 
   keyfile::get_object_value(*this, &chroot::set_description,
 			    keyfile, get_name(), "description",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("description");
 
   keyfile::get_object_list_value(*this, &chroot::set_users,
 				 keyfile, get_name(), "users",
 				 keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("users");
 
   keyfile::get_object_list_value(*this, &chroot::set_groups,
 				 keyfile, get_name(), "groups",
 				 keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("groups");
 
   keyfile::get_object_list_value(*this, &chroot::set_root_users,
 				 keyfile, get_name(), "root-users",
 				 keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("root-users");
 
   keyfile::get_object_list_value(*this, &chroot::set_root_groups,
 				 keyfile, get_name(), "root-groups",
 				 keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("root-groups");
 
   keyfile::get_object_value(*this, &chroot::set_mount_location,
 			    keyfile, get_name(), "mount-location",
 			    get_active() ?
 			    keyfile::PRIORITY_REQUIRED :
 			    keyfile::PRIORITY_DISALLOWED);
+  used_keys.push_back("mount-location");
 
   keyfile::get_object_value(*this, &chroot::set_mount_device,
 			    keyfile, get_name(), "mount-device",
 			    get_active() ?
 			    keyfile::PRIORITY_OPTIONAL :
 			    keyfile::PRIORITY_DISALLOWED);
+  used_keys.push_back("mount-device");
 
   keyfile::get_object_list_value(*this, &chroot::set_command_prefix,
 				 keyfile, get_name(), "command-prefix",
 				 keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("command-prefix");
 
   keyfile::get_object_value(*this, &chroot::set_persona,
 			    keyfile, get_name(), "personality",
 			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("personality");
 }
